@@ -9,81 +9,86 @@ describe 'glance::api' do
 
   let :default_params do
     {
-      :bind_host                => '<SERVICE DEFAULT>',
-      :bind_port                => '9292',
-      :registry_host            => '0.0.0.0',
-      :registry_port            => '<SERVICE DEFAULT>',
-      :registry_client_protocol => '<SERVICE DEFAULT>',
-      :auth_strategy            => 'keystone',
-      :enabled                  => true,
-      :manage_service           => true,
-      :backlog                  => '<SERVICE DEFAULT>',
-      :workers                  => '7',
-      :show_image_direct_url    => '<SERVICE DEFAULT>',
-      :show_multiple_locations  => '<SERVICE DEFAULT>',
+      :bind_host                      => '<SERVICE DEFAULT>',
+      :bind_port                      => '9292',
+      :registry_host                  => '0.0.0.0',
+      :registry_port                  => '<SERVICE DEFAULT>',
+      :registry_client_protocol       => '<SERVICE DEFAULT>',
+      :auth_strategy                  => 'keystone',
+      :enabled                        => true,
+      :manage_service                 => true,
+      :backlog                        => '<SERVICE DEFAULT>',
+      :workers                        => '7',
+      :show_image_direct_url          => '<SERVICE DEFAULT>',
+      :show_multiple_locations        => '<SERVICE DEFAULT>',
       :filesystem_store_metadata_file => '<SERVICE DEFAULT>',
-      :filesystem_store_file_perm => '<SERVICE DEFAULT>',
-      :location_strategy        => '<SERVICE DEFAULT>',
-      :purge_config             => false,
-      :delayed_delete           => '<SERVICE DEFAULT>',
-      :scrub_time               => '<SERVICE DEFAULT>',
-      :default_store            => false,
-      :image_cache_dir          => '/var/lib/glance/image-cache',
-      :image_import_plugins     => '<SERVICE DEFAULT>',
+      :filesystem_store_file_perm     => '<SERVICE DEFAULT>',
+      :location_strategy              => '<SERVICE DEFAULT>',
+      :purge_config                   => false,
+      :delayed_delete                 => '<SERVICE DEFAULT>',
+      :scrub_time                     => '<SERVICE DEFAULT>',
+      :image_cache_dir                => '/var/lib/glance/image-cache',
+      :image_import_plugins           => '<SERVICE DEFAULT>',
       :image_conversion_output_format => '<SERVICE DEFAULT>',
-      :enabled_import_methods   => '<SERVICE DEFAULT>',
-      :node_staging_uri         => '<SERVICE DEFAULT>',
-      :image_member_quota       => '<SERVICE DEFAULT>',
-      :image_cache_stall_time   => '<SERVICE DEFAULT>',
-      :image_cache_max_size     => '<SERVICE DEFAULT>',
-      :os_region_name           => 'RegionOne',
-      :pipeline                 => 'keystone',
-      :task_time_to_live        => '<SERVICE DEFAULT>',
-      :task_executor            => '<SERVICE DEFAULT>',
-      :task_work_dir            => '<SERVICE DEFAULT>',
-      :taskflow_engine_mode     => '<SERVICE DEFAULT>',
-      :taskflow_max_workers     => '<SERVICE DEFAULT>',
-      :conversion_format        => '<SERVICE DEFAULT>',
-      :enable_v1_api            => false,
-      :enable_v2_api            => '<SERVICE DEFAULT>',
-      :sync_db                  => true,
-      :limit_param_default      => '<SERVICE DEFAULT>',
-      :api_limit_max            => '<SERVICE DEFAULT>',
+      :inject_metadata_properties     => '<SERVICE DEFAULT>',
+      :ignore_user_roles              => '<SERVICE DEFAULT>',
+      :enabled_import_methods         => '<SERVICE DEFAULT>',
+      :node_staging_uri               => '<SERVICE DEFAULT>',
+      :image_member_quota             => '<SERVICE DEFAULT>',
+      :image_cache_stall_time         => '<SERVICE DEFAULT>',
+      :image_cache_max_size           => '<SERVICE DEFAULT>',
+      :cache_prefetcher_interval      => '<SERVICE DEFAULT>',
+      :os_region_name                 => 'RegionOne',
+      :pipeline                       => 'keystone',
+      :task_time_to_live              => '<SERVICE DEFAULT>',
+      :task_executor                  => '<SERVICE DEFAULT>',
+      :task_work_dir                  => '<SERVICE DEFAULT>',
+      :taskflow_engine_mode           => '<SERVICE DEFAULT>',
+      :taskflow_max_workers           => '<SERVICE DEFAULT>',
+      :conversion_format              => '<SERVICE DEFAULT>',
+      :enable_v1_api                  => false,
+      :enable_v2_api                  => '<SERVICE DEFAULT>',
+      :sync_db                        => true,
+      :limit_param_default            => '<SERVICE DEFAULT>',
+      :api_limit_max                  => '<SERVICE DEFAULT>',
     }
   end
 
   shared_examples_for 'glance::api' do
 
     [{
-        :bind_host                => '127.0.0.1',
-        :bind_port                => '9222',
-        :registry_host            => '127.0.0.1',
-        :registry_port            => '9111',
-        :registry_client_protocol => 'https',
-        :auth_strategy            => 'not_keystone',
-        :enabled                  => false,
-        :backlog                  => '4095',
-        :workers                  => '5',
-        :show_image_direct_url    => true,
-        :show_multiple_locations  => true,
+        :bind_host                      => '127.0.0.1',
+        :bind_port                      => '9222',
+        :registry_host                  => '127.0.0.1',
+        :registry_port                  => '9111',
+        :registry_client_protocol       => 'https',
+        :auth_strategy                  => 'not_keystone',
+        :enabled                        => false,
+        :backlog                        => '4095',
+        :workers                        => '5',
+        :show_image_direct_url          => true,
+        :show_multiple_locations        => true,
         :filesystem_store_metadata_file => '/etc/glance/glance-metadata-file.conf',
-        :filesystem_store_file_perm => '0644',
-        :location_strategy        => 'store_type',
-        :delayed_delete           => 'true',
-        :scrub_time               => '10',
-        :image_cache_dir          => '/tmp/glance',
-        :image_import_plugins     => 'image_conversion',
+        :filesystem_store_file_perm     => '0644',
+        :location_strategy              => 'store_type',
+        :delayed_delete                 => 'true',
+        :scrub_time                     => '10',
+        :image_cache_dir                => '/tmp/glance',
+        :image_import_plugins           => 'image_conversion',
         :image_conversion_output_format => 'raw',
-        :enabled_import_methods   => 'glance-direct,web-download',
-        :node_staging_uri         => '/tmp/staging',
-        :image_member_quota       => '128',
-        :image_cache_stall_time   => '10',
-        :image_cache_max_size     => '10737418240',
-        :os_region_name           => 'RegionOne2',
-        :pipeline                 => 'keystone2',
-        :sync_db                  => false,
-        :limit_param_default      => '10',
-        :api_limit_max            => '10',
+        :inject_metadata_properties     => 'key:val',
+        :ignore_user_roles              => 'admin',
+        :enabled_import_methods         => 'glance-direct,web-download',
+        :node_staging_uri               => '/tmp/staging',
+        :image_member_quota             => '128',
+        :image_cache_stall_time         => '10',
+        :image_cache_max_size           => '10737418240',
+        :cache_prefetcher_interval      => '300',
+        :os_region_name                 => 'RegionOne2',
+        :pipeline                       => 'keystone2',
+        :sync_db                        => false,
+        :limit_param_default            => '10',
+        :api_limit_max                  => '10',
       }
     ].each do |param_set|
 
@@ -132,6 +137,9 @@ describe 'glance::api' do
             'delayed_delete',
             'scrub_time',
             'image_cache_dir',
+            'image_cache_stall_time',
+            'image_cache_max_size',
+            'cache_prefetcher_interval',
             'node_staging_uri',
             'image_member_quota',
             'enable_v1_api',
@@ -155,18 +163,18 @@ describe 'glance::api' do
                            "[%s]" % param_hash[:image_import_plugins])
         end
 
-        it 'is_expected.to lay down default image_conversion image_import config' do
-          [
-            'image_conversion_output_format'
-          ].each do |config|
-            is_expected.to contain_glance_image_import_config("image_conversion/output_format").with_value(param_hash[config.intern])
-          end
+
+        it 'is_expected.to lay down default image_conversion & inject_metadata image_import config' do
+          is_expected.to contain_glance_image_import_config("image_conversion/output_format").with_value(param_hash[:image_conversion_output_format])
+          is_expected.to contain_glance_image_import_config("inject_metadata_properties/inject").with_value(param_hash[:inject_metadata_properties])
+          is_expected.to contain_glance_image_import_config("inject_metadata_properties/ignore_user_roles").with_value(param_hash[:ignore_user_roles])
         end
 
         it 'is_expected.to lay down default cache config' do
           [
             'registry_host',
             'registry_port',
+            'image_cache_dir',
             'image_cache_stall_time',
             'image_cache_max_size',
           ].each do |config|
@@ -271,7 +279,7 @@ describe 'glance::api' do
         end
 
         it { expect { is_expected.to contain_glance_api_config('filter:paste_deploy/flavor') }.to\
-          raise_error(Puppet::Error, /validate_re\(\): .* does not match/) }
+          raise_error(Puppet::Error) }
       end
     end
 
@@ -285,6 +293,16 @@ describe 'glance::api' do
       )}
     end
 
+    describe 'setting max_request_body_size' do
+      let :params do
+        default_params.merge({:max_request_body_size => '102400' })
+      end
+
+      it { is_expected.to contain_oslo__middleware('glance_api_config').with(
+        :max_request_body_size => '102400',
+      )}
+    end
+
     describe 'with ssl options' do
       let :params do
         default_params.merge({
@@ -307,14 +325,74 @@ describe 'glance::api' do
       end
     end
 
-    describe 'with stores by default' do
+    describe 'with enabled_backends and stores by default' do
       let :params do
         default_params
       end
 
+      it { is_expected.to_not contain_glance_api_config('DEFAULT/enabled_backends').with_value('<SERVICE DEFAULT>') }
       it { is_expected.to_not contain_glance_api_config('glance_store/stores').with_value('<SERVICE DEFAULT>') }
     end
 
+    describe 'with enabled_backends' do
+      let :params do
+        default_params.merge({
+          :enabled_backends => ['file1:file','http1:http'],
+          :default_backend  => 'file1',
+          :stores           => ['file','http'],
+          :default_store    => 'file',
+        })
+      end
+
+      it { is_expected.to contain_glance_api_config('DEFAULT/enabled_backends').with_value('file1:file,http1:http') }
+      it { is_expected.to contain_glance_api_config('glance_store/default_backend').with_value('file1') }
+      it { is_expected.to contain_glance_api_config('glance_store/stores').with_ensure('absent') }
+      it { is_expected.to contain_glance_api_config('glance_store/default_store').with_ensure('absent') }
+    end
+
+    describe 'with invalid backend type' do
+      let :params do
+        default_params.merge({
+          :enabled_backends => ['file1:file','bad1:mybad'],
+          :default_backend  => 'file',
+        })
+      end
+
+      it_raises 'a Puppet::Error', / is not a valid backend type./
+    end
+
+    describe 'with enabled_backends but no default_backend' do
+      let :params do
+        default_params.merge({
+          :enabled_backends => ['file1:file','http1:http'],
+        })
+      end
+
+      it_raises 'a Puppet::Error', /A glance default_backend must be specified./
+    end
+
+    describe 'with duplicate backend identifiers' do
+      let :params do
+        default_params.merge({
+          :enabled_backends => ['file1:file','file1:http'],
+          :default_backend  => 'file1',
+        })
+      end
+
+      it_raises 'a Puppet::Error', /All backend identifiers in enabled_backends must be unique./
+    end
+
+    describe 'with invalid default_backend' do
+      let :params do
+        default_params.merge({
+          :enabled_backends => ['file1:file','http1:http'],
+          :default_backend  => 'file2',
+        })
+      end
+
+      it_raises 'a Puppet::Error', / is not a valid backend identifier./
+    end
+
     describe 'with stores override' do
       let :params do
         default_params.merge({
